@@ -2,12 +2,10 @@
 exception classes and constants handling test outcomes
 as well as functions creating them
 """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import sys
 
+from packaging.version import Version
+
 
 class OutcomeException(BaseException):
     """ OutcomeException and its subclass instances indicate and
@@ -25,7 +23,7 @@ class OutcomeException(BaseException):
             if isinstance(val, bytes):
                 val = val.decode("UTF-8", errors="replace")
             return val
-        return "<%s instance>" % (self.__class__.__name__,)
+        return "<{} instance>".format(self.__class__.__name__)
 
     __str__ = __repr__
 
@@ -55,7 +53,7 @@ class Exit(Exception):
     def __init__(self, msg="unknown reason", returncode=None):
         self.msg = msg
         self.returncode = returncode
-        super(Exit, self).__init__(msg)
+        super().__init__(msg)
 
 
 # exposed helper methods
@@ -75,7 +73,7 @@ def exit(msg, returncode=None):
 exit.Exception = Exit
 
 
-def skip(msg="", **kwargs):
+def skip(msg="", *, allow_module_level=False):
     """
     Skip an executing test with the given message.
 
@@ -95,9 +93,6 @@ def skip(msg="", **kwargs):
         to skip a doctest statically.
     """
     __tracebackhide__ = True
-    allow_module_level = kwargs.pop("allow_module_level", False)
-    if kwargs:
-        raise TypeError("unexpected keyword arguments: {}".format(sorted(kwargs)))
     raise Skipped(msg=msg, allow_module_level=allow_module_level)
 
 
@@ -119,7 +114,7 @@ def fail(msg="", pytrace=True):
 fail.Exception = Failed
 
 
-class XFailed(fail.Exception):
+class XFailed(Failed):
     """ raised from an explicit call to pytest.xfail() """
 
 
@@ -154,7 +149,6 @@ def importorskip(modname, minversion=None, reason=None):
 
     __tracebackhide__ = True
     compile(modname, "", "eval")  # to catch syntaxerrors
-    should_skip = False
 
     with warnings.catch_warnings():
         # make sure to ignore ImportWarnings that might happen because
@@ -163,27 +157,16 @@ def importorskip(modname, minversion=None, reason=None):
         warnings.simplefilter("ignore")
         try:
             __import__(modname)
-        except ImportError:
-            # Do not raise chained exception here(#1485)
-            should_skip = True
-    if should_skip:
-        if reason is None:
-            reason = "could not import %r" % (modname,)
-        raise Skipped(reason, allow_module_level=True)
+        except ImportError as exc:
+            if reason is None:
+                reason = "could not import {!r}: {}".format(modname, exc)
+            raise Skipped(reason, allow_module_level=True) from None
     mod = sys.modules[modname]
     if minversion is None:
         return mod
     verattr = getattr(mod, "__version__", None)
     if minversion is not None:
-        try:
-            from pkg_resources import parse_version as pv
-        except ImportError:
-            raise Skipped(
-                "we have a required version for %r but can not import "
-                "pkg_resources to parse version strings." % (modname,),
-                allow_module_level=True,
-            )
-        if verattr is None or pv(verattr) < pv(minversion):
+        if verattr is None or Version(verattr) < Version(minversion):
             raise Skipped(
                 "module %r has __version__ %r, required is: %r"
                 % (modname, verattr, minversion),
