@@ -5,23 +5,16 @@ import itertools
 import operator
 import os
 import shutil
-import stat
 import sys
 import uuid
-from functools import reduce
 from os.path import expanduser
 from os.path import expandvars
 from os.path import isabs
 from os.path import sep
 from posixpath import sep as posix_sep
 
-import six
-from six.moves import map
 
-from .compat import PY36
-
-
-if PY36:
+if sys.version_info[:2] >= (3, 6):
     from pathlib import Path, PurePath
 else:
     from pathlib2 import Path, PurePath
@@ -36,24 +29,17 @@ get_lock_path = operator.methodcaller("joinpath", ".lock")
 
 def ensure_reset_dir(path):
     """
-    ensures the given path is a empty directory
+    ensures the given path is an empty directory
     """
     if path.exists():
         rmtree(path, force=True)
     path.mkdir()
 
 
-def _shutil_rmtree_remove_writable(func, fspath, _):
-    "Clear the readonly bit and reattempt the removal"
-    os.chmod(fspath, stat.S_IWRITE)
-    func(fspath)
-
-
 def rmtree(path, force=False):
     if force:
-        # ignore_errors leaves dead folders around
-        # python needs a rm -rf as a followup
-        # the trick with _shutil_rmtree_remove_writable is unreliable
+        # NOTE: ignore_errors might leave dead folders around.
+        #       Python needs a rm -rf as a followup.
         shutil.rmtree(str(path), ignore_errors=True)
     else:
         shutil.rmtree(str(path))
@@ -92,22 +78,11 @@ def parse_num(maybe_num):
         return -1
 
 
-if six.PY2:
-
-    def _max(iterable, default):
-        """needed due to python2.7 lacking the default argument for max"""
-        return reduce(max, iterable, default)
-
-
-else:
-    _max = max
-
-
 def _force_symlink(root, target, link_to):
     """helper to create the current symlink
 
-    its full of race conditions that are reasonably ok to ignore
-    for the contex of best effort linking to the latest testrun
+    it's full of race conditions that are reasonably ok to ignore
+    for the context of best effort linking to the latest testrun
 
     the presumption being thatin case of much parallelism
     the inaccuracy is going to be acceptable
@@ -124,10 +99,10 @@ def _force_symlink(root, target, link_to):
 
 
 def make_numbered_dir(root, prefix):
-    """create a directory with a increased number as suffix for the given prefix"""
+    """create a directory with an increased number as suffix for the given prefix"""
     for i in range(10):
         # try up to 10 times to create the folder
-        max_existing = _max(map(parse_num, find_suffixes(root, prefix)), default=-1)
+        max_existing = max(map(parse_num, find_suffixes(root, prefix)), default=-1)
         new_number = max_existing + 1
         new_path = root.joinpath("{}{}".format(prefix, new_number))
         try:
@@ -151,20 +126,19 @@ def create_cleanup_lock(p):
         fd = os.open(str(lock_path), os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0o644)
     except OSError as e:
         if e.errno == errno.EEXIST:
-            six.raise_from(
-                EnvironmentError("cannot create lockfile in {path}".format(path=p)), e
-            )
+            raise EnvironmentError(
+                "cannot create lockfile in {path}".format(path=p)
+            ) from e
+
         else:
             raise
     else:
         pid = os.getpid()
-        spid = str(pid)
-        if not isinstance(spid, bytes):
-            spid = spid.encode("ascii")
+        spid = str(pid).encode()
         os.write(fd, spid)
         os.close(fd)
         if not lock_path.is_file():
-            raise EnvironmentError("lock path got renamed after sucessfull creation")
+            raise EnvironmentError("lock path got renamed after successful creation")
         return lock_path
 
 
@@ -186,19 +160,29 @@ def register_cleanup_lock_removal(lock_path, register=atexit.register):
 
 
 def maybe_delete_a_numbered_dir(path):
-    """removes a numbered directory if its lock can be obtained"""
+    """removes a numbered directory if its lock can be obtained and it does not seem to be in use"""
+    lock_path = None
     try:
-        create_cleanup_lock(path)
+        lock_path = create_cleanup_lock(path)
+        parent = path.parent
+
+        garbage = parent.joinpath("garbage-{}".format(uuid.uuid4()))
+        path.rename(garbage)
+        rmtree(garbage, force=True)
     except (OSError, EnvironmentError):
         #  known races:
         #  * other process did a cleanup at the same time
         #  * deletable folder was found
+        #  * process cwd (Windows)
         return
-    parent = path.parent
-
-    garbage = parent.joinpath("garbage-{}".format(uuid.uuid4()))
-    path.rename(garbage)
-    rmtree(garbage, force=True)
+    finally:
+        # if we created the lock, ensure we remove it even if we failed
+        # to properly remove the numbered dir
+        if lock_path is not None:
+            try:
+                lock_path.unlink()
+            except (OSError, IOError):
+                pass
 
 
 def ensure_deletable(path, consider_lock_dead_if_created_before):
@@ -221,14 +205,14 @@ def ensure_deletable(path, consider_lock_dead_if_created_before):
 
 
 def try_cleanup(path, consider_lock_dead_if_created_before):
-    """tries to cleanup a folder if we can ensure its deletable"""
+    """tries to cleanup a folder if we can ensure it's deletable"""
     if ensure_deletable(path, consider_lock_dead_if_created_before):
         maybe_delete_a_numbered_dir(path)
 
 
 def cleanup_candidates(root, prefix, keep):
     """lists candidates for numbered directories to be removed - follows py.path"""
-    max_existing = _max(map(parse_num, find_suffixes(root, prefix)), default=-1)
+    max_existing = max(map(parse_num, find_suffixes(root, prefix)), default=-1)
     max_delete = max_existing - keep
     paths = find_prefixed(root, prefix)
     paths, paths2 = itertools.tee(paths)
@@ -309,5 +293,12 @@ def fnmatch_ex(pattern, path):
     if sep not in pattern:
         name = path.name
     else:
-        name = six.text_type(path)
+        name = str(path)
+        if path.is_absolute() and not os.path.isabs(pattern):
+            pattern = "*{}{}".format(os.sep, pattern)
     return fnmatch.fnmatch(name, pattern)
+
+
+def parts(s):
+    parts = s.split(sep)
+    return {sep.join(parts[: i + 1]) or sep for i in range(len(parts))}
